from mqtttasky_groupme import gpio_status_light_util as status_light
from datetime import datetime as dt, timedelta
from mqtttasky_groupme.config import mqtt_config
from PIL import Image as PILImage, ImageTk
from cryptography.fernet import Fernet
from threading import Thread
from time import sleep
from tkinter import *

import paho.mqtt.client as mqtt
import pkg_resources
import os

mqtt_client = mqtt.Client()

key = ''
cipher = None

MESSAGE_ENCODING = 'utf-8'

# Taken from https://raspberrypi.stackexchange.com/questions/38294/error-when-attempting-to-create-python-gui-using-tkinter-no-display-name-and-n
# Necessary when the $DISPLAY var is null:
if os.environ.get('DISPLAY','') == '':
    print('No display found. Using :0.0')
    os.environ.__setitem__('DISPLAY', ':0.0')


# This is called on by app.py because all of the config data must be entered before it is used..
def init_cipher():
    global key
    global cipher
    key = mqtt_config.CONFIG['cipher_key']
    cipher = Fernet(key)


# This is also called on by app.py because all of the config data must be entered before it is used..
def init_mqtt_client():
    # Use the username and password pair as provided in the
    # MQTT configuration:
    mqtt_client.username_pw_set(mqtt_config.CONFIG['username'],
                            mqtt_config.CONFIG['password'])

    mqtt_client.connect(host=mqtt_config.CONFIG['host'],
                    port=mqtt_config.CONFIG['port']) # Connect using host/port from config
    print('\n[ MQTT Client ] Connected to broker.\n')

    try:
        enc_task_buffer = cipher.encrypt(bytes('Hello from MQ T T Tasky!', MESSAGE_ENCODING))
        mqtt_client.publish('pub/tasks', enc_task_buffer)
    except:
        pass
    Thread(target=mqtt_client.loop_forever).start()

# The task object represents a task and is used as an arg
# to TaskWindow to display it graphically.
class Task:
    def __init__(self, title, start_date, task_time,
                    notif_time, description):
        self.title       = title
        self.start_date  = start_date
        self.task_time   = task_time
        self.description = description

        task_dt = dt.strptime((start_date + ' ' + task_time)
                                , '%m/%d/%y %H:%M')

        # The below code takes a time like '30 minutes before'
        # and creates a datetime object that would actually
        # represent that amount of time before the task's start
        # date and time.
        #
        # If format invalid or not recognized, set notification
        # time to equal task time (notification will be given
        # when the task is set to start).
        if ' minute' in notif_time:
            try:
                delta = notif_time.split(' minute')[0]
                notif_time_dt = task_dt - timedelta(minutes=int(delta))
                self.notif_time = notif_time_dt
            except:
                self.notif_time = task_dt
        elif ' hour' in notif_time:
            try:
                delta = notif_time.split( 'hour')[0]
                notif_time_dt = task_dt - timedelta(hours=int(delta))
                self.notif_time = notif_time_dt
            except:
                self.notif_time = task_dt
        elif ' day' in notif_time:
            try:
                delta = notif_time.split(' day')[0]
                notif_time_dt = task_dt - timedelta(days=int(delta))
                self.notif_time = notif_time_dt
            except:
                self.notif_time = task_dt
        elif ' week' in notif_time:
            try:
                delta = notif_time.split(' week')[0]
                notif_time_dt = task_dt - timedelta(weeks=int(delta))
                self.notif_time = notif_time_dt
            except:
                self.notif_time = task_dt
        else:
            self.notif_time = task_dt


    # This function is used to generate a string to be read by
    # text-to-speech engine simple_google_tts
    def tts(self):
        return (
            f"Task { self.title } starts on { self.start_date }"
            f" at { self.task_time }. Task's description is as follows:"
            f" { self.description }"
        )


    def __str__(self):
        if len(self.description) > 60:
            desc = '{}'.format(self.description[:60]) # Truncate to 60 chars.
        else:
            desc = self.description
        return (
            f"Title:        { self.title }"
            f"\nStart Date:  { self.start_date }"
            f"\nTask Time:   { self.task_time }"
            f"\nNotif. Time: { self.notif_time }"
            f"\nDescription: { desc }"
        )


# Singleton class -- learned from https://python-3-patterns-idioms-test.readthedocs.io/en/latest/Singleton.html
class TaskScheduler:
    class __TaskScheduler:
        def __init__(self):
            self.task_list = []
            self.iterable_task_list = []
            t = Thread(target=self.manage_notifications)
            t.start()


        def schedule(self, task):
            self.task_list.append(task)
            print(task.notif_time)


        def remove(self, task):
            self.task_list.remove(task)

        
        def remove_all(self):
            try: # May fail if it is being iterated over.
                self.task_list.clear()
                return True
            except:
                sleep(.2)
                try:
                    self.task_list.clear()
                    return True
                except:
                    return False


        def pub_notification(self, task_str):
            Thread(target=status_light.blink_notification).start() # Blink lights in sequence.
            try:
                # Convert the message from utf-8 string to byte array and encrypt:
                enc_task_buffer = cipher.encrypt(bytes(task_str, MESSAGE_ENCODING))
                # Publish
                mqtt_client.publish('pub/tasks', enc_task_buffer)
            except:
                pass

        # Call on simple_google_tts to read the task aloud:
        def read_task(self, task_str):
            os.system('simple_google_tts -p en ' + '\"' + task_str + '\"')


        # This function will iterate over each task in list and compare
        # its notification time to the current time. When they match,
        # it broadcasts a notification to pub/tasks, reads the message
        # aloud using simple_google_tts, displays it graphically and
        # deletes the Task object.
        def manage_notifications(self):
            while True:
                for t in self.task_list:
                    self.iterable_task_list.append(t)

                now = dt.now().replace(second=0, microsecond=0)
                for t in self.iterable_task_list:
                    if t.notif_time == now:
                        pub_thread = Thread(target=self.pub_notification, args=(t.tts(),))
                        tts_thread = Thread(target=self.read_task, args=(t.tts(),))
                        gui_thread = Thread(target=TaskWindow, args=(t,))

                        pub_thread.start() # Encrypt and publish to pub/tasks
                        tts_thread.start() # Read aloud over using simple_google_tts
                        gui_thread.start() # Display the Task on screen for several seconds
                        
                        self.task_list.remove(t)

                self.iterable_task_list.clear()
                sleep(30)

    instance = None


    def __new__(cls):
        if not TaskScheduler.instance:
            TaskScheduler.instance = TaskScheduler.__TaskScheduler()
        return TaskScheduler.instance


    def __getattr__(self, name):
        return getattr(self.instance, name)


    def __setattr__(self, name):
        return setattr(self.instance, name)
        
# Basic GUI window, loads the data into labels, adds the labels
# to the window, displays for 4 seconds, and self-disposes.
class TaskWindow:
    def __init__(self, task):
        self.title       = task.title
        self.start_date  = task.start_date
        self.task_time   = task.task_time
        self.description = task.description

        self.WINDOW = Tk()
        self.WINDOW.title('Task alert: ' + self.title)
        self.WINDOW.minsize(400, 300)

        self.init_components()
        self.init_labels()
        self.add_to_window()

        self.WINDOW.after(4000, lambda: self.WINDOW.destroy())
        mainloop()


    def init_components(self):
        path = pkg_resources.resource_filename(__name__, 'images/clock-32x32.png')
        self.clock_image = ImageTk.PhotoImage(
            PILImage.open(path)
        )


    def init_labels(self):
        self.title_label   = Label(self.WINDOW,
                                    text=self.title,
                                    fg='#4286F6',
                                    font=('Arial', 26),
                                    wraplength=350)

        self.image_label   = Label(self.WINDOW,
                                    image=self.clock_image)

        self.header_label1 = Label(self.WINDOW,
                                    text='Today',
                                    fg='black',
                                    font=('Arial', 16))

        self.header_label2 = Label( self.WINDOW,
                                    text=(self.start_date),
                                    fg='black',
                                    font=('Arial', 16))

        self.time_label    = Label(self.WINDOW,
                                    text=self.task_time,
                                    fg='black',
                                    font=('Arial', 14))

        self.descr_label   = Label(self.WINDOW,
                                    text=self.description,
                                    fg='black',
                                    bg = '#E6E6E6',
                                    font=('Arial', 18),
                                    highlightbackground='#4286F6',
                                    highlightthickness=1,
                                    wraplength=350)

    def add_to_window(self):
        self.title_label.place(x=32, y=16)
        self.image_label.place(x=32, y=108)
        self.header_label1.place(x=76, y=92)
        self.header_label2.place(x=156, y=92)
        self.time_label.place(x=76, y=120)
        self.descr_label.place(x=32, y=180)



from mqtttasky_groupme import gpio_status_light_util as status_light
from mqtttasky_groupme import bot_calls
from mqtttasky_groupme import config_util
from mqtttasky_groupme import tasks
from threading import Thread
from time import sleep

import pkg_resources
import requests
import re


config_data = []
# Join the package's path with the data path to access config.
configfile = pkg_resources.resource_filename(__name__, 'config/groupme_bot_config.py')
with open(configfile, 'r') as f:
    for i, line in enumerate(f):
        # Parse user's string and store in config_data[]:
        if ('=' in line):
            split_str = line.split('=', 1)
            var = split_str[1].replace("'", '')
            config_data.append(re.sub(r'#.*', '', var).strip())

# Copy groupme data from file:
ACCESS_TOKEN = config_data[0]
BOT_ID       = config_data[1]
GROUP_ID     = config_data[2]
BOT_NAME     = config_data[3]
GROUP_NAME   = config_data[4]

# Now that symmetric has been generated, initialized the Fernet cipher
tasks.init_cipher()
tasks.init_mqtt_client()

POST_URL = 'https://api.groupme.com/v3/bots/post' # URL for POST requests
RESP_URL = 'https://api.groupme.com/v3/groups/' + GROUP_ID + '/messages' # URL to read messages from

request_params = { 'token': ACCESS_TOKEN }


def send_reply(reply, message):
    post_params = {
        'bot_id': BOT_ID,
        'text': reply
    }
    req = requests.post(POST_URL, params=post_params)
    request_params['since_id'] = message['id'] # Mark with since_id so previous messages aren't replied to.

    print(message['text'] + '\n')
    print('Sent reply: ' + reply)
    print(req) # Print the status code in response to the POST request.

# Check messages in group every 4 seconds, when a new message arrives, if it
# Does not come from a bot, convert it to lowercase and pass it to bot_calls'
# generate_reply function to process message and send reply back to user using
# send_reply function above.
def main():
    while True:
        try:
            response = requests.get(RESP_URL, params=request_params)

            if response.status_code == 200:
                response_messages = response.json()['response']['messages']
                
                # Indicate messages retreived okay:
                Thread(target=status_light.blink_successful_retrieval).start()
                
                # As long as this is printed, user knows bot is actively listening to group.
                print('\n#######################' +
                    '\n# Bot: ' + BOT_NAME + 
                    '\n# Group: ' + GROUP_NAME + 
                    '\n# Status: Listening' + 
                    '\n#######################')

                for message in response_messages:
                    if not (message['sender_type'] == 'bot'): # Bot doesn't reply to itself (infinite loop!).
                        # Indicate new message received:
                        Thread(target=status_light.blink_incoming).start()
                        if not (message['text'] == None):
                            send_reply(bot_calls.generate_reply(message['text'].lower()), message)
                            # Indicate message being replied to:
                            Thread(target=status_light.blink_outgoing).start()
                            break

                # Indicate message is being sent:
                Thread(target=status_light.blink_outgoing).start()
            else:
                print('Message retrieval failed: ' + response.status_code)
                # Indicate that messages were not retrieved:
                Thread(target=status_light.blink_unsuccessful_retrieval).start()

        except requests.exceptions.ConnectionError as e:
            print('No connection')
            print(e)

        except requests.exceptions.Timeout as e:
            print('Connection timed out')
            print(e)

        except requests.exceptions.RequestException as e:
            print('A request error ocurred')
            print(e)

        except Exception as e:
            print('An error occurred')
            print(e)

        sleep(4)

main()



from mqtttasky_groupme.tasks import Task, TaskScheduler
from datetime import timedelta, datetime as dt

import random
import re

# Represents data needed to schedule a single task:
task_data = {
    'title': '',
    'description': '',
    'start_date': '',
    'task_time': '',
    'notification': ''
}

title_asked = False
notif_asked = False

notif_time = '30 minutes before'
time_pattern = re.compile("^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$") # Pattern to validate time strings
notif_pattern = re.compile("[0-9]?[0-9] [a-z*]") # Pattern to validate time strings for notification

task_scheduler = TaskScheduler()

JOKES = [
    ' you were born...',
    '\nQ: What do you call a fish without an eye?\n\nA: A fsh.',
    ' I went to a restaurant on the moon the other day, the food' +
    ' was great but there was just no atmosphere',
    ' two guys walked into a bar, I just walked underneath.',
    ' the 3 unwritten rules of life:\n   1:\n   2:\n   3:',
    ' what do you and a bungalow have in common?\nNothing upstairs!',
    ' my for-loops take longer to end than your relationships',
    ' I\'m an acquired taste. if you don\'t like me, acquire some taste.',
    ' If I had a face like yours, I\'d sue my parents',
]

PROVERBS = [
    'The too-lazy do a few things poorly; the too-busy do many things' +
    ' poorly.',
    'If you store food in your fridge, it may go bad. If you store food in' +
    ' your belly, you don\'t need a fridge.',
    'When you are lacking in motivation, you are lacking in caffeine',
    'Once you exceed your life limit, you cannot upgrade your plan.',
    'Nothing is in control and there\'s nothing you can do about it.',
    'There are 2 types of people: those who can interpret incomplete data' +
    ' sets.',
    '\"Give a man a fish and feed him for a day.\nDon\'t teach a man to fish' +
    ' and feed yourself.\nHe\'s a grown man and fishing\'s not that hard.\" - Ron'
]


def reset_task_data_et_questions():
    global task_data
    global title_asked
    global notif_asked

    task_data = {
        'title': '',
        'description': '',
        'start_date': '',
        'task_time': '',
        'notification': ''
    }
    title_asked = False
    notif_asked = False


# Ensures task date is not already in past
def validate_date(date):
    dt_date = dt.strptime(date, '%m/%d/%y')
    now = dt.now()
    diff = dt_date - now
    if diff.days < -1: # Return false if date precedes today's date.
        print(diff.days)
        return False
    else:
        return True


# Validates the date format and checks to see whether the date
# is in the past. If validation passed, the task's date is set accordingly.
# Reply string and date_valid bool returned.
def try_set_task_date(date):
    global task_data
    date_valid = False
    try:
        dt.strptime(date, '%m/%d/%y') # Validate date format
        if validate_date(date): # Validate the date isn't already past.
            task_data['start_date'] = date
            reply = 'Task date is set to ' + date
            date_valid = True
        else:
            reply = 'The selected date is already past'
    except:
        # Append current year if not specified:
        try:
            date = date + '/' + str(dt.now().year)[2:]
            dt.strptime(date, '%m/%d/%y') # Validate date format
            if validate_date(date): # Validate the date isn't already past.
                task_data['start_date'] = date
                reply = 'Task date is set to ' + date
                date_valid = True
            else:
                reply = 'The selected date is already past'
        except:
            reply = f"Date format is invalid. Use 'MM/DD' or 'MM/DD/YY'"

    return reply, date_valid


# Validates the time format passed by checking it against time_pattern.
# If AM/PM, it converts to 24-hr time. Sets the task's time if validation
# passed. Returns string reply and bool time_valid.
def try_set_task_time(time):
    global task_data
    reply = f"The task time is invalid. Format the time as 'HH:MM' or 'HH:MM pm'"
    time_valid = False

    if 'am' in time:
        time = time.split(' am')[0]
        if re.match(time_pattern, time): # Validate time
            task_data['task_time'] = time
            reply = 'Task time is set to ' + time
            time_valid = True

    elif 'pm' in time:
        time = time.split(' pm')[0]
        if re.match(time_pattern, time): # Validate time
            # Convert to 24-hr:
            time_split = time.split(':')
            hour_int = int(time_split[0]) + 12
            task_data['task_time'] = str(hour_int) + ':' + time_split[1]
            reply = 'Task time is set to ' + task_data['task_time']
            time_valid = True

    else: # Already in 24-hr time
        if re.match(time_pattern, time):
            task_data['task_time'] = time
            reply = 'Task time is set to ' + time
            time_valid = True

    return reply, time_valid


# Attempts to set the notification time of task. If the time
# string matches the pattern, notif_pattern and the time unit
# used (minute, hour, day, etc.) is recognized, the task_data's
# notification value is set to the string.
# Returns string reply and bool time_valid
def try_set_notif_time(time):
    global task_data

    if re.match(notif_pattern, time): # Check that the pattern matches
        if ((' minute' in time) or (' hour' in time) or (' day' in time)
            or (' week' in time)): # check for one of the valid units of time
            task_data['notification'] = time
            reply = 'Notification set for ' + time
            time_valid = True
    elif 'start time' in time:
        task_data['notification'] = time
        reply = 'Notification set for ' + time
        time_valid = True
    else:
        reply = (
            f"The notification duration is invalid. E.g. '45 minutes before', "
            f" '2 days before', or 'at the task's start time'."
        )
        time_valid = False

    return reply, time_valid


# Processes contents of msg and replies with a pertinent repsponse
# such as the next instruction. Uses regex and simple string matching
# to decide what information is being given to parse data into task_data
# and give appropriate response, etc.
def generate_reply(msg):
    global task_data
    global title_asked
    global notif_asked

    # If none of the outermost if/elif cases evaluate to true,
    # this response is given:
    reply = (
        f"I'm here to schedule your tasks."
        f" To schedule a new task, just say '"
        f"Schedule task'."
        f"\n\n For usage, type '/usage'"
    )

    if msg == '/usage': # Displays usage instructions.
        reply = (
            f"\n                 MQTTasky Usage"
            f"\n==============================="
            f"\n- Initiate a new task: "
            f"\n\t 'Schedule task'"
            f"\n\t 'Schedule task <title> with description <description>'"
            f"\n\n- Date and time:"
            f"\n\tMQTTTasky will let you know how"
            f"\n\tto enter date or time when prompted."
            f"\n\tIf the year is not provided in a date"
            f"\n\tstring, the current year is assumed."
            f"\n\n\tTime strings ending in AM or PM are"
            f"\n\tautomatically converted into 24-hr"
            f"\n\tformat."
            f"\n\n- Notifications:"
            f"\n\tNotifications are specified by a unit"
            f"\n\tof time before the time of the task."
            f"\n\tYou may use the following units of"
            f"\n\ttime:"
            f"\n\n\tMinutes:"
            f"\n\t\t E.g. '15 minutes before'"
            f"\n\tHours:"
            f"\n\t\t E.g. '15 hours before'"
            f"\n\tDays:"
            f"\n\t\t E.g. '15 days before'"
            f"\n\tWeeks:"
            f"\n\t\t E.g. '15 weeks before'"
            f"\n\n- Reset:"
            f"\n\tIf you want to cancel or restart the"
            f"\n\tcreation of a task, use the command:"
            f"\n\t\t '/reset'"
            f"\n\n- Delete:"
            f"\n\tTo delete all active tasks:"
            f"\n\t\t '/deleteall'"
            f"\n\n- Bonus:"
            f"\n\tAsk bot to tell a joke or proverb."
        )
    elif msg == '/reset': # Clears task_data thereby allowing user to start over with task scheduling.
        reset_task_data_et_questions()
        reply = 'Task data cleared.'
    elif msg == '/deleteall': # Clear the task_list in task_scheduler
        removed = task_scheduler.remove_all()
        if removed:
            reply = 'All scheduled tasks have been deleted.'
        else:
            reply = 'Deletion of scheduled tasks failed. Please try again.'
    elif re.search('tell.*joke', msg): # Tell joke from jokes array:
        reply = JOKES[random.randint(0, ( len(JOKES) - 1 ))]
        print(reply)
    elif re.search('tell.*proverb', msg): # Tell proverb from proverbs array:
        reply = PROVERBS[random.randint(0, ( len(PROVERBS) - 1 ))]
    elif re.search('give.*wisdom', msg): # Tell proverb from proverbs array:
        reply = PROVERBS[random.randint(0, ( len(PROVERBS) - 1 ))]

    elif task_data['title'] == '': # If title string empty, check for new task scheduled
        if re.search('schedule.*task .* with description .*', msg): # Try to parse title and desc to task_data
            split_str = msg.split(' task ')[1]
            split = split_str.split(' with description ')
            task_data['title'] = split[0][0].upper() + split[0][1:] # Capitalize first char
            task_data['description'] = split[1]
            reply = (
                f"On what date will '{ task_data['title'] }' occur?"
                f" Use 'MM/DD' or 'MM/DD/YY'"
            )

        elif re.search('schedule.*task', msg): # Prompt for title
            reply = 'What should be the new task\'s title?'
            title_asked = True
        
        elif title_asked: # After title has been given, this response will be parsed as title:
            task_data['title'] = msg[0].upper() + msg[1:]
            reply = 'The task\'s title is set to ' + task_data['title']
            reply = f"Enter the description for the task \'{ task_data['title'] }'"

    # Title is already given; check if description needed:
    elif task_data['description'] == '':
        task_data['description'] = msg
        # Set description and ask for date
        reply = (
            f"The task\'s description is set to '{ msg }'"
            f". On what date will the task '{ task_data['title'] }' occur?"
            f" Use 'MM/DD' or 'MM/DD/YY'"
        )

    # Title and description given, parse the date for task to start on:
    elif task_data['start_date'] == '':
        reply, date_valid = try_set_task_date(msg)
        reply += f". At what time will the task '{ task_data['title'] }' occur?"
        reply += " Use 'HH:mm' or 'HH:mm AM/PM'"

    # Title, description, and date parsed to task_data. Get the time
    # that task occurs at:
    elif task_data['task_time'] == '':
        reply, time_valid = try_set_task_time(msg)
        notif_str = (
            f". You have the default notification time set to { notif_time }" 
            f" before the task occurs. Would you like to change it?"
        )
        reply += notif_str

    # Title, description, date, and start_time parsed. Get the notification
    # string indicating when before the task start time a notification should
    # be given:
    elif task_data['notification'] == '':        
        if not notif_asked:
            # if user replied with yes or y, they don't want default notification time
            # of 30 minutes before. let them change it.
            if ('yes' in msg) or (msg == 'y'):
                reply = (
                    f"When should the notification occur? E.g. '45 minutes before' or"
                    f" '2 days before', or 'at the task's start time'."
                )
                notif_asked = True
            # Else, event is ready to be scheduled. Send reply re-iterating task data
            # and schedule the task by first creating a task object and then passing
            # it to task_scheduler's schedule function:
            else:
                try_set_notif_time(notif_time)
                if 'start time' in task_data['notification']:
                    reply = (
                        f"\n\nTask '{ task_data['title'] }' with description '{ task_data['description'] }'"
                        f" is set to occur at { task_data['task_time'] } on { task_data['start_date'] }."
                        f" You have set the notification to occur at the task's start time."
                    )
                else:
                    reply = (
                        f"\n\nTask '{ task_data['title'] }' with description '{ task_data['description'] }'"
                        f" is set to occur at { task_data['task_time'] } on { task_data['start_date'] }."
                        f" You have set the notification to occur { task_data['notification'] }."
                    )
                # Schedule task here
                t = Task(
                    task_data['title'], task_data['start_date'], task_data['task_time'],
                    task_data['notification'], task_data['description']
                )
                task_scheduler.schedule(t)
                reset_task_data_et_questions() # Reset for next task to be scheduled.
                
        else:
            reply, time_valid = try_set_notif_time(msg)
            if time_valid:
                # Time validated; Event is ready to be scheduled. Send reply re-iterating task data
                # and schedule the task by first creating a task object and then passing
                # it to task_scheduler's schedule function:
                if 'start time' in task_data['notification']:
                    reply = (
                        f"\n\nTask '{ task_data['title'] }' with description '{ task_data['description'] }'"
                        f" is set to occur at { task_data['task_time'] } on { task_data['start_date'] }."
                        f" You have set the notification to occur at the task's start time."
                    )
                else:
                    reply = (
                        f"\n\nTask '{ task_data['title'] }' with description '{ task_data['description'] }'"
                        f" is set to occur at { task_data['task_time'] } on { task_data['start_date'] }."
                        f" You have set the notification to occur { task_data['notification'] }."
                    )
                # Schedule task here
                t = Task(
                    task_data['title'], task_data['start_date'], task_data['task_time'],
                    task_data['notification'], task_data['description']
                )
                task_scheduler.schedule(t)
                reset_task_data_et_questions() # Reset for next task to be scheduled.

    return reply



from cryptography.fernet import Fernet

import pkg_resources

def main():
    config_data = []
    loop_input = True

    print('################ MQTTTasky Configuration ################')
    print('\nWelcome! Yes, MQTTTasky is a terrible name! If you haven\'t')
    print('\nalready, take the time to read README.rst before')
    print('\ncontinuing with setup.')
    usr_input = input('\n     Press Enter to continue or \'q\' to quit. >> ')

    if (usr_input.lower() == 'q'):
        exit(0)

    # Collect all user input
    while loop_input:
        print('\n ---------------- GroupMe Configuration ----------------')
        usr_input = input('1) Enter the your GroupMe access token >> ')
        config_data.append(usr_input)
        usr_input = input('2) Now enter your GroupMe bot ID >> ')
        config_data.append(usr_input)
        usr_input = input('3) Now enter your GroupMe group ID >> ')
        config_data.append(usr_input)

        print('\n ------------ Optional GroupMe Configuration ------------')
        usr_input = input('*Optional* Enter the name for your Bot (only' +
                        '\ndisplays in this program) >> ')
        if (len(usr_input) == 0):
                usr_input = 'MQTTTasky Bot'

        config_data.append(usr_input)

        usr_input = input('*Optional* Enter the name for your Group (only' +
                        '\ndisplays in this program) >> ')
        if (len(usr_input) == 0):
                usr_input = 'MQTTTasky Notifications'

        config_data.append(usr_input)

        print('\n ------------------ MQTT Configuration ------------------')
        usr_input = input('1) Enter your MQTT port (usually 1883) >> ')
        config_data.append(usr_input)
        usr_input = input('2) Enter your MQTT host address >> ')
        config_data.append(usr_input)
        usr_input = input('3) Enter your MQTT username >> ')
        config_data.append(usr_input)
        usr_input = input('4) Enter your MQTT password >> ')
        config_data.append(usr_input)

        missing_data = False
        # Check for missing data; if found, clear and start over
        for i in range(9):
            if (len(config_data[i]) == 0):
                # tell user first line left blank
                print('\nYou left one or more lines blank starting with' +
                    '\nline ' + str(i + 1) + '\n')
                # clear contents of array if user forgot to enter line
                # don't let code crash if user entered nothing
                if (len(config_data) > 0):
                    del(config_data[:])
                missing_data = True
                break

        if not (missing_data):
            loop_input = False # Assuming all needed data recorded; break out of loop to write to file.

    groupme_file = pkg_resources.resource_filename(__name__, 'config/groupme_bot_config.py')
    mqtt_file = pkg_resources.resource_filename(__name__, 'config/mqtt_config.py')
    # Write recorded groupme info to file:
    with open(groupme_file, 'w') as f:
        f.write('ACCESS_TOKEN = \'' + config_data[0] +
                    '\'     # GROUPME ACCESS TOKEN\n')

        f.write('BOT_ID = \'' + config_data[1] +
                    '\'         # GROUPME BOT ID\n')

        f.write('GROUP_ID = \'' + config_data[2] +
                    '\'       # GROUPME GROUP ID\n')

        f.write('BOT_NAME = \'' + config_data[3] +
                    '\'       # GROUPME BOT NAME (ARBITRARY)\n')

        f.write('GROUP_NAME = \'' + config_data[4] +
                    '\'     # GROUPME GROUP NAME (ARBITRARY)')

    cipher_key = Fernet.generate_key()
    key_str = cipher_key.decode('utf-8')
        
    # Write recorded mqtt info + fernet cipher key to file:
    with open(mqtt_file, 'w') as f:
        f.write('CONFIG = {')
        f.write('\n   \'port\': ' + config_data[5] + ',')
        f.write('\n   \'host\': \'' + config_data[6] + '\',')
        f.write('\n   \'username\': \'' + config_data[7] + '\',')
        f.write('\n   \'password\': \'' + config_data[8] + '\',')
        f.write('\n   \'cipher_key\': \'' + key_str + '\'')
        f.write('\n}')

    # Share cipher key so any client can use fernet algorithm to decrypt payloads:
    print('\nBytes published by the MQTT client are encrypted')
    print('using Fernet encryption. Record the following')
    print('cipher key and use it with subscribers to decrypt')
    print('MQTTTasky\'s payloads:')
    print('\n' + key_str)
    print('\n\nThe program must be restarted. Re-run')
    print('\'mqttasky_groupme\' to restart it.')
    input('\nPress enter to continue...')
    exit(0)

main()


from time import sleep

import RPi.GPIO as gpio
import atexit

atexit.register(gpio.cleanup) # Properly dispose of gpio resources

RED_PIN    = 18
YELLOW_PIN = 23
GREEN_PIN  = 24

blinking = False

gpio.setmode(gpio.BCM)
gpio.setup(RED_PIN, gpio.OUT)
gpio.setup(YELLOW_PIN, gpio.OUT)
gpio.setup(GREEN_PIN, gpio.OUT)


# Blink pattern to indicate that messages were successfully retreived:
def blink_successful_retrieval():
    global blinking
    if blinking: # Wait for current use of LEDs to finish
        sleep(.8)

    blinking = True

    gpio.output(YELLOW_PIN, gpio.LOW)
    sleep(.8)
    gpio.output(YELLOW_PIN, gpio.HIGH)

    blinking = False


# Blink pattern to indicate that messages were not successfully retreived:
def blink_unsuccessful_retrieval():
    global blinking
    if blinking: # Wait for current use of LEDs to finish
        sleep(.8)

    blinking = True

    gpio.output(RED_PIN, gpio.LOW)
    sleep(.8)
    gpio.output(RED_PIN, gpio.HIGH)

    blinking = False


# Blink pattern to indicate that an new message was received:
def blink_incoming():
    global blinking
    if blinking: # Wait for current use of LEDs to finish
        sleep(.8)

    blinking = True

    gpio.output(GREEN_PIN, gpio.LOW)
    sleep(.2)
    gpio.output(GREEN_PIN, gpio.HIGH)
    sleep(.1)
    gpio.output(GREEN_PIN, gpio.LOW)
    sleep(.2)
    gpio.output(GREEN_PIN, gpio.HIGH)
    sleep(.1)
    gpio.output(GREEN_PIN, gpio.LOW)
    sleep(.2)
    gpio.output(GREEN_PIN, gpio.HIGH)

    blinking = False
    

# Blink pattern to indicate that an outgoing message was sent:
def blink_outgoing():
    global blinking
    if blinking: # Wait for current use of LEDs to finish
        sleep(.8)

    blinking = True

    gpio.output(YELLOW_PIN, gpio.LOW)
    sleep(.2)
    gpio.output(YELLOW_PIN, gpio.HIGH)
    sleep(.1)
    gpio.output(YELLOW_PIN, gpio.LOW)
    sleep(.2)
    gpio.output(YELLOW_PIN, gpio.HIGH)
    sleep(.1)
    gpio.output(YELLOW_PIN, gpio.LOW)
    sleep(.2)
    gpio.output(YELLOW_PIN, gpio.HIGH)

    blinking = False


# Blink pattern to indicate that a task notification will be displayed:
def blink_notification():
    global blinking
    if blinking: # Wait for current use of LEDs to finish
        sleep(.8)

    blinking = True
    
    for i in range(0, 3):
        gpio.output(RED_PIN, gpio.LOW)
        sleep(.1)
        gpio.output(RED_PIN, gpio.HIGH)
        gpio.output(YELLOW_PIN, gpio.LOW)
        sleep(.1)
        gpio.output(YELLOW_PIN, gpio.HIGH)
        gpio.output(GREEN_PIN, gpio.LOW)
        sleep(.1)
        gpio.output(GREEN_PIN, gpio.HIGH)

    blinking = False


from cryptography.fernet import Fernet

import pkg_resources
import re

def main():
    config_data = []
    loop_input = True
    missing_data = False

    # Initialize all data necessary to run the program from config.

    # Join the package's path with the data path to access config.
    groupme_file = pkg_resources.resource_filename(__name__, 'config/groupme_bot_config.py')
    mqtt_file = pkg_resources.resource_filename(__name__, 'config/mqtt_config.py')
    # Check the groupme config file to see if it has all the values in it:
    with open(groupme_file, 'r') as f:
        for i, line in enumerate(f):
            # Parse user's string and store in config_data[]:
            if ('=' in line):
                split_str = line.split('=', 1)
                var = split_str[1].replace("'", '')
                config_data.append(re.sub(r'#.*', '', var).strip())

                if (len(config_data[i]) == 0):
                    missing_data = True

    # Check the mqtt config file to see if it has all the values in it:
    with open(mqtt_file, 'r') as f:
        for i, line in enumerate(f):
            if (':' in line):
                split_str = line.split(': ', 1)
                var = split_str[1].replace(",", '')
                try:
                    config_data.append(var)

                    if (len(config_data[i]) == 0):
                        missing_data = True
                except:
                    missing_data = True

    # If files are completely blank.
    if (len(config_data) == 0):
        missing_data = True

    # IF any missing data, collect all the necessary data from the beginning:
    if (missing_data):
        del(config_data[:])
        print('################ MQTTTasky Configuration ################')
        print('\nWelcome! Yes, MQTTTasky is a terrible name! If you haven\'t')
        print('\nalready, take the time to read README.rst before')
        print('\ncontinuing with setup.')
        usr_input = input('\n     Press Enter to continue or \'q\' to quit. >> ')

        if (usr_input.lower() == 'q'):
            exit(0)

        while loop_input:
            print('\n ---------------- GroupMe Configuration ----------------')
            usr_input = input('1) Enter the your GroupMe access token >> ')
            config_data.append(usr_input)
            usr_input = input('2) Now enter your GroupMe bot ID >> ')
            config_data.append(usr_input)
            usr_input = input('3) Now enter your GroupMe group ID >> ')
            config_data.append(usr_input)

            print('\n ------------ Optional GroupMe Configuration ------------')
            usr_input = input('*Optional* Enter the name for your Bot (only' +
                            '\ndisplays in this program) >> ')
            if (len(usr_input) == 0):
                    usr_input = 'MQTTTasky Bot'

            config_data.append(usr_input)

            usr_input = input('*Optional* Enter the name for your Group (only' +
                            '\ndisplays in this program) >> ')
            if (len(usr_input) == 0):
                    usr_input = 'MQTTTasky Notifications'

            config_data.append(usr_input)

            print('\n ------------------ MQTT Configuration ------------------')
            usr_input = input('1) Enter your MQTT port (usually 1883) >> ')
            config_data.append(usr_input)
            usr_input = input('2) Enter your MQTT host address >> ')
            config_data.append(usr_input)
            usr_input = input('3) Enter your MQTT username >> ')
            config_data.append(usr_input)
            usr_input = input('4) Enter your MQTT password >> ')
            config_data.append(usr_input)

            missing_data = False
            # Check once again to see if all the data has been entered:
            for i in range(9):
                if (len(config_data[i]) == 0):
                    # tell user first line left blank
                    print('\nYou left one or more lines blank starting with' +
                        '\nline ' + str(i + 1) + '\n')
                    # clear contents of array if user forgot to enter line
                    # don't let code crash if user entered nothing
                    if (len(config_data) > 0):
                        del(config_data[:])
                    missing_data = True
                    break

            if not (missing_data):
                loop_input = False # Assuming all the correct info has been supplied.

        # Write the recorded groupme data to groupme config file
        with open(groupme_file, 'w') as f:
            f.write('ACCESS_TOKEN = \'' + config_data[0] +
                        '\'     # GROUPME ACCESS TOKEN\n')

            f.write('BOT_ID = \'' + config_data[1] +
                        '\'         # GROUPME BOT ID\n')

            f.write('GROUP_ID = \'' + config_data[2] +
                        '\'       # GROUPME GROUP ID\n')

            f.write('BOT_NAME = \'' + config_data[3] +
                        '\'       # GROUPME BOT NAME (ARBITRARY)\n')

            f.write('GROUP_NAME = \'' + config_data[4] +
                        '\'     # GROUPME GROUP NAME (ARBITRARY)')

        cipher_key = Fernet.generate_key()
        key_str = cipher_key.decode('utf-8')

        # Write the recorded mqtt data to groupme config file + write the cipher key to be used:
        with open(mqtt_file, 'w') as f:
            f.write('CONFIG = {')
            f.write('\n   \'port\': ' + config_data[5] + ',')
            f.write('\n   \'host\': \'' + config_data[6] + '\',')
            f.write('\n   \'username\': \'' + config_data[7] + '\',')
            f.write('\n   \'password\': \'' + config_data[8] + '\',')
            f.write('\n   \'cipher_key\': \'' + key_str + '\'')
            f.write('\n}')

        # Share the cypher key so any client can use fernet to decrypt:
        print('\nBytes published by the MQTT client are encrypted')
        print('using Fernet encryption. Record the following')
        print('cipher key and use it with subscribers to decrypt')
        print('MQTTTasky\'s payloads:')
        print('\n' + key_str)
        print('\n\nThe program must be restarted.')
        print('Press Ctrl-C to exit')
        exit(1)

main()


#! /usr/bin/env python

from setuptools import setup, find_packages
from setuptools.command.install import install

with open("README.rst", "r") as readme:
    long_desc = readme.read()

setup(
    name='mqtttasky_groupme',
    version='0.1.4',
    license='MIT',
    author='Ryan Haas',
    author_email='haasrr@etsu.edu',
    description='GroupMe bot to schedule tasks, display and speak notifications, and publish them over MQTT.',
    long_description=long_desc,
    long_description_content_type='text/x-rst',
    url='https://github.com/haasr/mqtttttttasky_groupme',
    packages=find_packages(),
    classifiers=[
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3 :: Only'
    ],
    entry_points={
        'console_scripts': [
            'mqtttasky_groupme=mqtttasky_groupme.app:main',
            'mqtttasky_groupme_config=mqtttasky_groupme.reconfig_util:main'
        ],
    },
    package_data={
        'mqtttasky_groupme': ['config/groupme_bot_config.py',
                                'config/mqtt_config.py', 'images/clock-32x32.png']
    },
    include_package_data=True,
    zip_safe=False,
    python_requires='>=3.7',
    install_requires=[
        'certifi==2020.11.8',
        'cffi==1.14.4',
        'chardet==3.0.4',
        'cryptography==3.2.1',
        'idna==2.10',
        'paho-mqtt==1.5.1',
        'Pillow==7.2.0',
        'pkg-resources==0.0.0',
        'pycparser==2.20',
        'requests==2.25.0',
        'RPi.GPIO==0.7.0',
        'six==1.15.0',
        'urllib3==1.26.2'
    ]
)